// Edgardo Molina
// File: MainWindow.cpp

#include <QtGui>
#include <cstdlib>

#include "MainWindow.h"
#include "ImagePanel.h"
#include "ops.h"


MainWindow::MainWindow() {
	
  // Initialize widgets and layout
  QWidget *w;
  QGridLayout *grid;
  
  _view  = new ImagePanel;
  
  _buttonBox = new QGroupBox("View:");
  QPushButton *origB   = new QPushButton("Original", _buttonBox);
  QPushButton *redB    = new QPushButton("Red Channel", _buttonBox);
  QPushButton *greenB  = new QPushButton("Green Channel", _buttonBox);
  QPushButton *blueB   = new QPushButton("Blue Channel", _buttonBox);
  QPushButton *grayB   = new QPushButton("Average gray", _buttonBox);
  QPushButton *lumB    = new QPushButton("Luminance", _buttonBox);
  QPushButton *diffB   = new QPushButton("Difference from Original", _buttonBox);
  _buttonBox->setEnabled(false);
  _buttonBox->setFixedWidth(200);
  
  // Add buttons to a vertical box
  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(origB);
  layout->addWidget(redB);
  layout->addWidget(greenB);
  layout->addWidget(blueB);
  layout->addWidget(grayB);
  layout->addWidget(lumB);
  layout->addWidget(diffB);

  _buttonBox->setLayout(layout);

  // Add buttons to a group, for internal purposes
  QButtonGroup *buttonGroup = new QButtonGroup(this);
  buttonGroup->addButton(origB, Original);
  buttonGroup->addButton(redB, RedChannel);
  buttonGroup->addButton(greenB, GreenChannel);
  buttonGroup->addButton(blueB, BlueChannel);
  buttonGroup->addButton(grayB, AverageGray);
  buttonGroup->addButton(lumB, Luminance);
  buttonGroup->addButton(diffB, OriginalDiff);

  // Connect a button-clicked to the operation function
  connect(buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(operation(int)));

  // Arrange widgets for display
  w = new QWidget;
  grid = new QGridLayout;
  grid->addWidget(_view, 0, 0);
  grid->addWidget(_buttonBox, 0, 2);
  grid->setRowStretch(0, 1);
  
  // Create file menus
  createActions();
  createMenus();
  
  setWindowTitle(tr("Capstone Example"));
  resize(800, 600);
  
  w->setLayout(grid);
  setCentralWidget(w);
}

// Called by File->Open or shortcut keys
void
MainWindow::open() {
  _file = QFileDialog::getOpenFileName(this, tr("Select a file"),
					 QDir::currentPath(), tr("Images (*.png *.xpm *.jpg *.bmp)"));
  
  if(!_file.isEmpty()) {
    _originalI = new QImage(_file);
    _currentI = new QImage(_file);
    _view->show(_currentI);
    _buttonBox->setEnabled(true); 
  }
}

// Called by File->Save or shortcut keys
void
MainWindow::save() {
    QString s = QFileDialog::getSaveFileName(
                                             this,
                                             "Choose a filename to save under",
                                             QDir::homePath(),
                                             "Images (*.bmp)");
    _currentI->save(s, "bmp");
}


void
MainWindow::about() {
  QMessageBox::about(this, tr("about Capstone Example"),
                     tr("<p>Capstone example by:<br />Edgardo Molina</p>"));
}


void
MainWindow::createActions() {
  _openAct = new QAction(tr("&Open..."), this);
  _openAct->setShortcut(tr("Ctrl+O"));
  connect(_openAct, SIGNAL(triggered()), this, SLOT(open()));
  
  _saveAct = new QAction(tr("&Save..."), this);
  _saveAct->setShortcut(tr("CTRL+S"));
  connect(_saveAct, SIGNAL(triggered()), this, SLOT(save()));
  
  _exitAct = new QAction(tr("E&xit"), this);
  _exitAct->setShortcut(tr("Ctrl+Q"));
  connect(_exitAct, SIGNAL(triggered()), this, SLOT(close()));
  
  _aboutAct = new QAction(tr("&About"), this);
  connect(_aboutAct, SIGNAL(triggered()), this, SLOT(about()));
  
  _aboutQtAct = new QAction(tr("About &Qt"), this);
  connect(_aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void 
MainWindow::createMenus() {
  _fileMenu = new QMenu(tr("&File"), this);
  _fileMenu->addAction(_openAct);
  _fileMenu->addAction(_saveAct);
  _fileMenu->addSeparator();
  _fileMenu->addAction(_exitAct);
  
  _helpMenu = new QMenu(tr("&Help"), this);
  _helpMenu->addAction(_aboutAct);
  _helpMenu->addAction(_aboutQtAct);
  
  menuBar()->addMenu(_fileMenu);
  menuBar()->addMenu(_helpMenu);
}

// Operation Slot
void
MainWindow::operation(int op) {
    QImage *I;
    switch (op) {
        case Original:
            I = new QImage(_file);
            break;
        case RedChannel:
            I = getChannel(_originalI, Red);
            break;
        case GreenChannel:
            I = getChannel(_originalI, Green);
            break;
        case BlueChannel:
            I = getChannel(_originalI, Blue);
            break;
        case AverageGray:
            I = toGrayscale(_originalI);
            break;
        case Luminance:
            I = toLuminance(_originalI);
            break;
        case OriginalDiff:
            I = diff(_currentI, _originalI);
            break;
    };
    delete _currentI;
    _currentI = I;
    _view->show(_currentI);
}
